package fyloz.trial.ColorRecipesExplorer.web.controller.editors;

import fyloz.trial.ColorRecipesExplorer.model.Recipe;
import fyloz.trial.ColorRecipesExplorer.services.CompanyService;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.*;
import static fyloz.trial.ColorRecipesExplorer.web.StringBank.*;

@Controller
public class RecipeEditorController {

    private RecipeService recipeService;
    private CompanyService companyService;

    @Autowired
    public RecipeEditorController(RecipeService recipeService, CompanyService companyService) {
        this.recipeService = recipeService;
        this.companyService = companyService;
    }

    /**
     * Affiche la page listant toutes les recettes.
     * <p>
     * Modèle de la page:
     * - recipes: Contient un Map de toutes les recettes triées par compagnie
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_RECIPE)
    public String listRecipes(Model model) {
        model.addAttribute(RECIPES, recipeService.getRecipesByCompany());

        return EDITOR_RECIPE;
    }

    /**
     * Affiche la page d'édition d'une recette.
     * Cette méthode requiert l'identifiant de la recette dans l'URL.
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipe: Contient la recette
     * - recipeJSON: Contient la recette sous forme de JSON
     * - companies: Contient la liste des compagnies
     * - mixes: Contient la liste des mélanges de la recettes, classées
     * - images: Contient la liste des noms des images liées à la recette
     *
     * @param model    Le Model injecté par Thymeleaf
     * @param recipeID L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_RECIPE_SPECIFIC)
    public String showEditPage(Model model, @PathVariable int recipeID) {
        Recipe recipe = recipeService.getByID(recipeID);

        if (recipe == null) {
            model.addAttribute(RESPONSE_ERROR, String.format(RECIPE_NOT_FOUND, recipeID));
            return listRecipes(model);
        }

        model.addAttribute(RECIPE, recipe);
        model.addAttribute("recipeJSON", recipeService.asJson(recipe));
        model.addAttribute(COMPANIES, companyService.getAll());
        model.addAttribute(MIXES, recipeService.getSortedMixes(recipe));
        model.addAttribute("images", recipeService.getImageFiles(recipe));

        return EDITOR_RECIPE_EDITOR;
    }

    /**
     * Permet à l'utilisateur de modifier une recette.
     * <p>
     * La mise à jour échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipeCode: Contient la couleur d'une recette
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param model  Le Model injecté par Thymeleaf
     * @param recipe La recette à modifier
     * @param form   Le formulaire entré par l'utilisateur
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_RECIPE, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public String saveRecipe(Model model, @Valid Recipe recipe, @RequestBody MultiValueMap<String, Object> form) {
        int recipeID = recipe.getRecipeID();

        if (recipeService.getByID(recipeID) == null) {
            model.addAttribute(RESPONSE_ERROR, String.format(RECIPE_NOT_FOUND, recipeID));
            return listRecipes(model);
        }

        recipe = recipeService.createAndSetSteps(recipe, form);

        model.addAttribute(RECIPE_CODE, recipe.getRecipeCode());
        return listRecipes(model);
    }
}
